@@ -30,13 +30,22 @@ You will need PIL/Pillow to use the screenshot features.
 from __future__ import absolute_import, division, print_function
 
 
-__version__ = '0.9.39'
+__version__ = '0.9.42'
 
-import collections
 import sys
 import time
 
 
+if sys.version_info[0] == 2 or sys.version_info[0:2] in ((3, 1), (3,2)):
+    # Python 2 and 3.1 and 3.2 uses collections.Sequence
+    import collections
+    collectionsSequence = collections.Sequence
+else:
+    # Python 3.3+ uses collections.abc.Sequence
+    import collections.abc
+    collectionsSequence = collections.abc.Sequence
+
+
 try:
     import pytweening
     from pytweening import (easeInQuad, easeOutQuad, easeInOutQuad,
@@ -50,7 +59,15 @@ try:
     # getPointOnLine has been redefined in this file, to avoid dependency on pytweening.
     # linear has also been redefined in this file.
 except ImportError:
-    pass
+    def couldNotImportPyTweening():
+        raise Exception('PyAutoGUI was unable to import pytweening. Please install this module.')
+        easeInQuad = easeOutQuad = easeInOutQuad = \
+        easeInCubic = easeOutCubic = easeInOutCubic = easeInQuart = easeOutQuart = \
+        easeInOutQuart = easeInQuint = easeOutQuint = easeInOutQuint = easeInSine = \
+        easeOutSine = easeInOutSine = easeInExpo = easeOutExpo = easeInOutExpo = \
+        easeInCirc = easeOutCirc = easeInOutCirc = easeInElastic = easeOutElastic = \
+        easeInOutElastic = easeInBack = easeOutBack = easeInOutBack = easeInBounce = \
+        easeOutBounce = easeInOutBounce = couldNotImportPyTweening
 
 
 try:
@@ -58,7 +75,9 @@ try:
     from pymsgbox import alert, confirm, prompt, password
 except ImportError:
     # If pymsgbox module is not found, those methods will not be available.
-    pass
+    def couldNotImportPyMsgBox():
+        raise Exception('PyAutoGUI was unable to import pymsgbox. Please install this module.')
+    alert = confirm = prompt = password = couldNotImportPyMsgBox
 
 
 try:
@@ -67,9 +86,28 @@ try:
         locateCenterOnScreen, locateOnScreen, pixel, pixelMatchesColor,
         screenshot)
 except ImportError:
-    # If pyscreeze module is not found, screenshot-related features will simply
-    # not work.
-    pass
+    # If pyscreeze module is not found, screenshot-related features will simply not work.
+    def couldNotImportPyScreeze():
+        raise Exception('PyAutoGUI was unable to import pyscreeze. Please install this module.')
+    center = grab = locate = locateAll = locateAllOnScreen = locateCenterOnScreen = locateOnScreen = pixel = pixelMatchesColor = screenshot = couldNotImportPyScreeze
+
+
+def useImageNotFoundException(value=None):
+    if value is None:
+        value = True
+    pyscreeze.USE_IMAGE_NOT_FOUND_EXCEPTION = value
+
+
+
+if sys.platform == 'win32': # PyGetWindow currently only supports Windows.
+    try:
+        import pygetwindow
+        from pygetwindow import Window, getFocusedWindow, getWindowsAt, getWindowsWithTitle, getAllWindows, getAllTitles
+    except ImportError:
+        # If pygetwindow module is not found, those methods will not be available.
+        def couldNotImportPyGetWindow():
+            raise Exception('PyAutoGUI was unable to import pygetwindow. Please install this module.')
+        Window = getFocusedWindow = getWindowsAt = getWindowsWithTitle = getAllWindows = getAllTitles = couldNotImportPyGetWindow
 
 
 KEY_NAMES = ['\t', '\n', '\r', ' ', '!', '"', '#', '$', '%', '&', "'", '(',
@@ -110,7 +148,6 @@ elif sys.platform == 'darwin':
     from . import _pyautogui_osx as platformModule
 elif sys.platform == 'win32':
     from . import _pyautogui_win as platformModule
-    from ._window_win import Window, getWindows, getWindow
 else:
     from . import _pyautogui_x11 as platformModule
 
@@ -124,12 +161,16 @@ else:
 # In seconds. Any duration less than this is rounded to 0.0 to instantly move
 # the mouse.
 MINIMUM_DURATION = 0.1
-# If sleep_amount is too short, time.sleep() will be a no-op and the mouse
-# cursor moves there instantly.
+# If sleep_amount is less than MINIMUM_DURATION, time.sleep() will be a no-op and the mouse cursor moves there instantly.
 # TODO: This value should vary with the platform. http://stackoverflow.com/q/1133857
 MINIMUM_SLEEP = 0.05
 PAUSE = 0.1 # The number of seconds to pause after EVERY public function call. Useful for debugging.
 FAILSAFE = True
+FAILSAFE_POINT = (0, 0) # If the mouse is here and FAILSAFE is True, the FailSafeException is raised.
+
+
+Point = collections.namedtuple('Point', 'x y')
+Size = collections.namedtuple('Size', 'width height')
 
 
 # General Functions
@@ -151,6 +192,8 @@ def linear(n):
 
     Copied from pytweening module.
     """
+
+    # We use this function instead of pytweening.linear for the default tween function just in case pytweening couldn't be imported.
     if not 0.0 <= n <= 1.0:
         raise ValueError('Argument must be between 0.0 and 1.0.')
     return n
@@ -170,7 +213,11 @@ def _unpackXY(x, y):
     On functions that receive a pair of x,y coordinates, they can be passed as
     separate arguments, or as a single two-element sequence.
     """
-    if isinstance(x, collections.Sequence):
+    if isinstance(x, str):
+        # x parameter is the string of an image filename to find and click on:
+        x, y = center(locateOnScreen(x))
+
+    elif isinstance(x, collectionsSequence):
         if len(x) == 2:
             if y is None:
                 x, y = x
@@ -179,7 +226,7 @@ def _unpackXY(x, y):
         else:
             raise ValueError('The supplied sequence must have exactly 2 elements ({0} were received).'.format(len(x)))
     else:
-        pass
+        pass # x and y are just number values
 
     return x, y
 
@@ -200,11 +247,11 @@ def position(x=None, y=None):
     posx, posy = platformModule._position()
     posx = int(posx)
     posy = int(posy)
-    if x is not None:
+    if x is not None: # If set, the x parameter overrides the return value.
         posx = int(x)
-    if y is not None:
+    if y is not None: # If set, the y parameter overrides the return value.
         posy = int(y)
-    return posx, posy
+    return Point(posx, posy)
 
 
 def size():
@@ -213,7 +260,7 @@ def size():
     Returns:
       (width, height) tuple of the screen size, in pixels.
     """
-    return platformModule._size()
+    return Size(*platformModule._size())
 
 
 def onScreen(x, y=None):
@@ -251,6 +298,8 @@ def mouseDown(x=None, y=None, button='left', duration=0.0, tween=linear, pause=N
     Args:
       x (int, float, None, tuple, optional): The x position on the screen where the
         mouse down happens. None by default. If tuple, this is used for x and y.
+        If x is a str, it's considered a filename of an image to find on
+        the screen with locateOnScreen() and click the center of.
       y (int, float, None, optional): The y position on the screen where the
         mouse down happens. None by default.
       button (str, int, optional): The mouse button pressed down. Must be one of
@@ -268,6 +317,7 @@ def mouseDown(x=None, y=None, button='left', duration=0.0, tween=linear, pause=N
 
     _failSafeCheck()
     x, y = _unpackXY(x, y)
+
     _mouseMoveDrag('move', x, y, 0, 0, duration=0, tween=None)
 
     x, y = platformModule._position() # TODO - this isn't right. We need to check the params.
@@ -292,6 +342,8 @@ def mouseUp(x=None, y=None, button='left', duration=0.0, tween=linear, pause=Non
     Args:
       x (int, float, None, tuple, optional): The x position on the screen where the
         mouse up happens. None by default. If tuple, this is used for x and y.
+        If x is a str, it's considered a filename of an image to find on
+        the screen with locateOnScreen() and click the center of.
       y (int, float, None, optional): The y position on the screen where the
         mouse up happens. None by default.
       button (str, int, optional): The mouse button released. Must be one of
@@ -309,6 +361,7 @@ def mouseUp(x=None, y=None, button='left', duration=0.0, tween=linear, pause=Non
 
     _failSafeCheck()
     x, y = _unpackXY(x, y)
+
     _mouseMoveDrag('move', x, y, 0, 0, duration=0, tween=None)
 
     x, y = platformModule._position()
@@ -330,8 +383,10 @@ def click(x=None, y=None, clicks=1, interval=0.0, button='left', duration=0.0, t
     screen.
 
     Args:
-      x (int, float, None, tuple, optional): The x position on the screen where
+      x (int, float, None, tuple, str, optional): The x position on the screen where
         the click happens. None by default. If tuple, this is used for x and y.
+        If x is a str, it's considered a filename of an image to find on
+        the screen with locateOnScreen() and click the center of.
       y (int, float, None, optional): The y position on the screen where the
         click happens. None by default.
       clicks (int, optional): The number of clicks to perform. 1 by default.
@@ -354,6 +409,7 @@ def click(x=None, y=None, clicks=1, interval=0.0, button='left', duration=0.0, t
 
     _failSafeCheck()
     x, y = _unpackXY(x, y)
+
     _mouseMoveDrag('move', x, y, 0, 0, duration, tween)
 
     x, y = platformModule._position()
@@ -386,6 +442,8 @@ def rightClick(x=None, y=None, duration=0.0, tween=linear, pause=None, _pause=Tr
     Args:
       x (int, float, None, tuple, optional): The x position on the screen where the
         click happens. None by default. If tuple, this is used for x and y.
+        If x is a str, it's considered a filename of an image to find on
+        the screen with locateOnScreen() and click the center of.
       y (int, float, None, optional): The y position on the screen where the
         click happens. None by default.
 
@@ -412,6 +470,8 @@ def middleClick(x=None, y=None, duration=0.0, tween=linear, pause=None, _pause=T
     Args:
       x (int, float, None, tuple, optional): The x position on the screen where the
         click happens. None by default. If tuple, this is used for x and y.
+        If x is a str, it's considered a filename of an image to find on
+        the screen with locateOnScreen() and click the center of.
       y (int, float, None, optional): The y position on the screen where the
         click happens. None by default.
 
@@ -438,6 +498,8 @@ def doubleClick(x=None, y=None, interval=0.0, button='left', duration=0.0, tween
     Args:
       x (int, float, None, tuple, optional): The x position on the screen where the
         click happens. None by default. If tuple, this is used for x and y.
+        If x is a str, it's considered a filename of an image to find on
+        the screen with locateOnScreen() and click the center of.
       y (int, float, None, optional): The y position on the screen where the
         click happens. None by default.
       interval (float, optional): The number of seconds in between each click,
@@ -481,6 +543,8 @@ def tripleClick(x=None, y=None, interval=0.0, button='left', duration=0.0, tween
     Args:
       x (int, float, None, tuple, optional): The x position on the screen where the
         click happens. None by default. If tuple, this is used for x and y.
+        If x is a str, it's considered a filename of an image to find on
+        the screen with locateOnScreen() and click the center of.
       y (int, float, None, optional): The y position on the screen where the
         click happens. None by default.
       interval (float, optional): The number of seconds in between each click,
@@ -609,6 +673,8 @@ def moveTo(x=None, y=None, duration=0.0, tween=linear, pause=None, _pause=True):
     Args:
       x (int, float, None, tuple, optional): The x position on the screen where the
         click happens. None by default. If tuple, this is used for x and y.
+        If x is a str, it's considered a filename of an image to find on
+        the screen with locateOnScreen() and click the center of.
       y (int, float, None, optional): The y position on the screen where the
         click happens. None by default.
       duration (float, optional): The amount of time it takes to move the mouse
@@ -621,9 +687,8 @@ def moveTo(x=None, y=None, duration=0.0, tween=linear, pause=None, _pause=True):
     Returns:
       None
     """
-    x, y = _unpackXY(x, y)
-
     _failSafeCheck()
+    x, y = _unpackXY(x, y)
 
     _mouseMoveDrag('move', x, y, 0, 0, duration, tween)
 
@@ -655,15 +720,18 @@ def moveRel(xOffset=None, yOffset=None, duration=0.0, tween=linear, pause=None,
       None
     """
 
-    xOffset, yOffset = _unpackXY(xOffset, yOffset)
-
     _failSafeCheck()
 
+    xOffset, yOffset = _unpackXY(xOffset, yOffset)
+
     _mouseMoveDrag('move', None, None, xOffset, yOffset, duration, tween)
 
     _autoPause(pause, _pause)
 
 
+move = moveRel # For PyAutoGUI 1.0, move() replaces moveRel().
+
+
 def dragTo(x=None, y=None, duration=0.0, tween=linear, button='left', pause=None, _pause=True, mouseDownUp=True):
     """Performs a mouse drag (mouse movement while a button is held down) to a
     point on the screen.
@@ -676,6 +744,8 @@ def dragTo(x=None, y=None, duration=0.0, tween=linear, button='left', pause=None
     Args:
       x (int, float, None, tuple, optional): How far left (for negative values) or
         right (for positive values) to move the cursor. 0 by default. If tuple, this is used for x and y.
+        If x is a str, it's considered a filename of an image to find on
+        the screen with locateOnScreen() and click the center of.
       y (int, float, None, optional): How far up (for negative values) or
         down (for positive values) to move the cursor. 0 by default.
       duration (float, optional): The amount of time it takes to move the mouse
@@ -694,8 +764,8 @@ def dragTo(x=None, y=None, duration=0.0, tween=linear, button='left', pause=None
       None
     """
     _failSafeCheck()
-    if type(x) in (tuple, list):
-        x, y = x[0], x[1]
+    x, y = _unpackXY(x, y)
+
     if mouseDownUp:
         mouseDown(button=button, _pause=False)
     _mouseMoveDrag('drag', x, y, 0, 0, duration, tween, button)
@@ -757,6 +827,9 @@ def dragRel(xOffset=0, yOffset=0, duration=0.0, tween=linear, button='left', pau
     _autoPause(pause, _pause)
 
 
+drag = dragRel # For PyAutoGUI 1.0, we want drag() to replace dragRel().
+
+
 def _mouseMoveDrag(moveOrDrag, x, y, xOffset, yOffset, duration, tween=linear, button=None):
     """Handles the actual move or drag event, since different platforms
     implement them differently.
@@ -988,6 +1061,9 @@ def typewrite(message, interval=0.0, pause=None, _pause=True):
     _autoPause(pause, _pause)
 
 
+write = typewrite # In PyAutoGUI 1.0, write() replaces typewrite().
+
+
 def hotkey(*args, **kwargs):
     """Performs key down presses on the arguments passed in order, then performs
     key releases in reverse order.
@@ -1027,7 +1103,22 @@ class FailSafeException(Exception):
 
 
 def _failSafeCheck():
-    if FAILSAFE and position() == (0, 0):
+    global FAILSAFE_POINT
+
+    if isinstance(FAILSAFE_POINT, str):
+        if FAILSAFE_POINT.lower() == 'topleft':
+            FAILSAFE_POINT = (0, 0)
+        elif FAILSAFE_POINT.lower() == 'topright':
+            FAILSAFE_POINT = (size()[0] - 1, 0)
+        elif FAILSAFE_POINT.lower() == 'bottomleft':
+            FAILSAFE_POINT = (0, size()[1] - 1)
+        elif FAILSAFE_POINT.lower() == 'bottomright':
+            screenSize = size()
+            FAILSAFE_POINT = (screenSize[0] - 1, screenSize[1] - 1)
+        else:
+            raise
+
+    if FAILSAFE and position() == FAILSAFE_POINT:
         raise FailSafeException('PyAutoGUI fail-safe triggered from mouse moving to upper-left corner. To disable this fail-safe, set pyautogui.FAILSAFE to False.')
 
 
